IMPLEMENTATION MODULE MultiSkelRgn;

FROM SYSTEM IMPORT ADDRESS;
FROM QuickDraw IMPORT Rect, Point, NewRgn, DisposeRgn, EraseRgn, EraseRect,
											UnionRgn, DiffRgn, SetEmptyRgn, EmptyRect, MapRgn,
											FrameRgn, GrafPtr, PenState, GetPenState, SetPenState,
											GetPort, PenPat, PenMode, gray, patXor, Pt2Rect,
											EqualPt, SectRect, RgnHandle, FrameRect, RectRgn;
FROM EventManager IMPORT GetDblTime, StillDown, GetMouse;
FROM MenuManager IMPORT EnableItem, DisableItem, DrawMenuBar;
FROM WindowManager IMPORT GetNewWindow, WindowPtr, CloseWindow;
FROM TransSkel IMPORT SkelWindow, WKeyProc, WCloseProc, WIdleProc;
FROM MultiSkelGlobals IMPORT rgnWindRes, shiftKey, SetWindClip, ResetWindClip,
														 DrawGrowBox, editMenu, rgnWind;


VAR
	selectRgn: RgnHandle;
	rgnPortRect: Rect;
	selectWhen: LONGINT;
	selectWhere: Point;


PROCEDURE RgnActivate(active: BOOLEAN);

	VAR
		i: INTEGER;

BEGIN
	DrawGrowBox(rgnWind);
	IF active THEN DisableItem(editMenu, 0);
	ELSE EnableItem(editMenu, 0); END;
	DrawMenuBar;
END RgnActivate;


PROCEDURE RgnDraw;

	VAR
		i: INTEGER;

BEGIN
	SetWindClip(rgnWind);
	FrameRgn(selectRgn);
	ResetWindClip;
END RgnDraw;

(*
	While mouse is down, draw gray selection rectangle in the current port.
	Return the resultant rect in dstRect.  The rect is always clipped to the
	current portRect.
*)
PROCEDURE DoSelectRect(startPoint: Point; VAR dstRect: Rect);

	VAR
		pt, dragPt: Point;
		rClip: Rect;
		thePort: GrafPtr;
		result: BOOLEAN;
		ps: PenState;
		i: INTEGER;

BEGIN
	GetPort(thePort);
	rClip := thePort^.portRect;
	DEC(rClip.right, 15);
	GetPenState(ps);
	PenPat(gray);
	PenMode(patXor);
	dragPt := startPoint;
	Pt2Rect(dragPt, dragPt, dstRect);
	FrameRect(dstRect);
	REPEAT
		GetMouse(pt);
		IF NOT EqualPt(pt, dragPt) THEN	(* Mouse has moved, change region *)
			FrameRect(dstRect);
			dragPt := pt;
			Pt2Rect(dragPt, startPoint, dstRect);
			result := SectRect(dstRect, rClip, dstRect);
			FrameRect(dstRect);
			FOR i := 0 TO 999 DO (* NOTHING *); END;
		END;
	UNTIL NOT StillDown();
	FrameRect(dstRect);
	SetPenState(ps);
END DoSelectRect;

(*
	On a dbl-click, clear window.  On a single click, draw gray selection rect
	as long as the mouse is down.  If the user draws a non-empty rect, then add
	it to the selection region and redraw the region's outline.  If the shiftkey
	was down, then subtract the selection and redraw.
*)

PROCEDURE RgnMouse(thePt: Point; t: LONGINT; mods: BITSET);

	VAR
		r: Rect;
		rgn: RgnHandle;
		
BEGIN
	r := rgnWind^.portRect;
	IF thePt.h >= r.right - 15 THEN RETURN; END;	(* Must not click in scroll area *)
	IF t - selectWhen <= GetDblTime() THEN	(* It's a double-click *)
		selectWhen := 0D;
		SetWindClip(rgnWind);
		EraseRgn(selectRgn);
		ResetWindClip;
		SetEmptyRgn(selectRgn);
	ELSE
		selectWhen := t;
		selectWhere := thePt;
		DoSelectRect(thePt, r);
		IF NOT EmptyRect(r) THEN
			selectWhen := 0D;
			rgn := NewRgn();
			RectRgn(rgn, r);
			IF shiftKey IN mods THEN DiffRgn(selectRgn, rgn, selectRgn);
			ELSE UnionRgn(selectRgn, rgn, selectRgn); END;
			DisposeRgn(rgn);
			SetWindClip(rgnWind);
			EraseRect(r);
			ResetWindClip;
			RgnDraw;
		END;
	END;
END RgnMouse;


PROCEDURE RgnUpdate(resized: BOOLEAN);

	VAR
		r: Rect;

BEGIN
	EraseRect(rgnWind^.portRect);
	IF resized THEN
		r := rgnWind^.portRect;
		DEC(rgnPortRect.right, 15);
		DEC(r.right, 15);
		MapRgn(selectRgn, rgnPortRect, r);
		rgnPortRect := rgnWind^.portRect;
	END;
	DrawGrowBox(rgnWind);
	RgnDraw;
END RgnUpdate;


PROCEDURE RgnHalt;

BEGIN
	DisposeRgn(selectRgn);
	CloseWindow(rgnWind);
END RgnHalt;


PROCEDURE RgnInit;

BEGIN
	rgnWind := GetNewWindow(rgnWindRes, VAL(ADDRESS, NIL), VAL(WindowPtr, -1D));
	SkelWindow(	rgnWind,
							RgnMouse,			(* Draw the rectangles in resp. to mouse downs *)
							VAL(WKeyProc, NIL),	(* Ignore key downs *)
							RgnUpdate,
							RgnActivate,
							VAL(WCloseProc, NIL),	(* No close proc *)
							RgnHalt,							(* Disposal procedure *)
							VAL(WIdleProc, NIL),	(* No idle proc *)
							TRUE);								(* Doesn't matter - no idle proc *)
	rgnPortRect := rgnWind^.portRect;
	selectRgn := NewRgn();						(* Selected region empty initially *)
	selectWhen := 0D;									(* 1st click can't be double-click *)
END RgnInit;

END MultiSkelRgn.
